@@ -168,6 +168,8 @@ class Model:
         static: Override the static value of the included model.
         pose: A position(x,y,z) and orientation(roll, pitch yaw) with
             respect to the frame named in the relative_to attribute.
+        plugin: A plugin is a dynamically loaded chunk of code. It can
+            exist as a child of world, model, and sensor.
         """
 
         uri: str = field(
@@ -201,6 +203,13 @@ class Model:
                 "required": True,
             },
         )
+        plugin: List["Model.Include.Plugin"] = field(
+            default_factory=list,
+            metadata={
+                "type": "Element",
+                "namespace": "",
+            },
+        )
 
         @dataclass
         class Pose:
@@ -226,6 +235,46 @@ class Model:
                 },
             )
 
+        @dataclass
+        class Plugin:
+            """A plugin is a dynamically loaded chunk of code.
+
+            It can exist as a child of world, model, and sensor.
+
+            Parameters
+            ----------
+            any_element: This is a special element that should not be
+                specified in an SDFormat file. It automatically copies
+                child elements into the SDFormat element so that a
+                plugin can access the data.
+            name: A unique name for the plugin, scoped to its parent.
+            filename: Name of the shared library to load. If the
+                filename is not a full path name, the file will be
+                searched for in the configuration paths.
+            """
+
+            any_element: List[object] = field(
+                default_factory=list,
+                metadata={
+                    "type": "Wildcard",
+                    "namespace": "##any",
+                },
+            )
+            name: Optional[str] = field(
+                default=None,
+                metadata={
+                    "type": "Attribute",
+                    "required": True,
+                },
+            )
+            filename: Optional[str] = field(
+                default=None,
+                metadata={
+                    "type": "Attribute",
+                    "required": True,
+                },
+            )
+
     @dataclass
     class Frame:
         """
